#GEE-based screening method (GEES)

#Arguments
#M --- n-dim vector containing #of obs for each subject
#X --- N*p design matrix
#Y --- N-dim responses
#corr --- working correlation: ind, exch, ar1
#link --- "Linear" linear model; "Logit" logistic regression; 
#         "Log" Poisson regression

#----------------------------------------------------------------

library(geepack)
library(MASS)

#----------------------------------------------------------------


#ind --- indices of potential important predictors 
#minimum model size (simulations)
MMS <- function(M, X, Y, corr, link, ind) {

#----------------------------------------------------------------

NGlob <- dim(X)[1]  #total number of obs.
nGlob <- length(M)  #total number of subjects
Id <- rep(NA, NGlob)   #used to divide the subjects
Id[1:M[1]] <- rep(1, M[1])
for(i in 2:nGlob) {
Id[(sum(M[1:(i-1)])+1):(sum(M[1:i]))] <- rep(i, M[i])
}

#----------------------------------------------------------------

if(link == "Linear") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
LM <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=LM, family=gaussian(), corstr=corr)
tau <- gee$alpha
}
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- min(abs(G0)[ind])
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

if(link == "Logit") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
Logit <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=Logit, family=binomial, corstr=corr)
tau <- gee$alpha
}
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- min(abs(G0)[ind])
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

if(link == "Log") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
Log <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=Log, family=poisson, corstr=corr)
tau <- gee$alpha
}
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- min(abs(G0)[ind])
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

result <- list(G0=G0, ind.beta=ind.beta)
return(result)

}


#----------------------------------------------------------------
#----------------------------------------------------------------


#d --- pre-specified model size
#tau --- estimate of parameters in R
#GEES method when d is given
Pd <- function(M, X, Y, d, tau, corr, link) {

#----------------------------------------------------------------

NGlob <- dim(X)[1]  #total number of obs.
pGlob <- dim(X)[2]  #number of predictors
nGlob <- length(M)  #total number of subjects

#----------------------------------------------------------------

if(link == "Linear") {
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- sort(abs(G0), decreasing=T)[d]
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

if(link == "Logit") {
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- sort(abs(G0), decreasing=T)[d]
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

if(link == "Log") {
G0 <- G0stat(M, X, Y, tau, corr, link) 
rn <- sort(abs(G0), decreasing=T)[d]
ind.beta <- which(abs(G0) >= rn)  #index of selected predictors
}

#----------------------------------------------------------------

result <- list(G0=G0, ind.beta=ind.beta)
return(result)

}


