#IGEES algorithm

#-----------------------------------------------------------------

library(geepack)
library(MASS)

#-----------------------------------------------------------------

#Argument:
#M --- n-dim vector containing #of obs for each subject
#X --- N*p design matrix
#Y --- N-dim responses
#link --- "Linear" linear model; "Logit" logistic regression; 
#         "Log" Poisson regression
#tau --- estimate of parameters in R
#corr --- working correlation: ind, exch, ar1
#d -- sparsity parameter size
#d1 -- size of A1 in Step 1

#-----------------------------------------------------------------

IGEES <- function(M,X,Y,link,tau,corr,d,d1) {

#----------------------------------------------------------------

NGlob <- dim(X)[1]
pGlob <- dim(X)[2]
nGlob <- length(M)  #total number of subjects
ini_dim <- 1:pGlob

#-----------------------------------------------------------------

################### Step 1 ####################
#select a subset A1
out1 <- Pd(M, X, Y, d1, tau, corr, link)
indd1 <- out1$ind.beta
print(indd1)

#----------------------------------------------------------------

################### Iteration ####################
while (d1 < d) {

#----------------------------------------------------------------

#predictor residual matrix
Projection <- X[,indd1]%*%solve(t(X[,indd1])%*%X[,indd1])%*%t(X[,indd1])
Xre <- (diag(NGlob)-Projection) %*% X[,-indd1]

#----------------------------------------------------------------

d2 <- min(d-d1, 5)# the size of set A2
#select a subset A2
out2 <- Pd(M, Xre, Y, d2, tau, corr, link)
indXre  <- out2$ind.beta
#the position of A2's features in  X
indd2 <- (ini_dim[-indd1])[indXre]
#the index of beta in step 2 
index <- sort(c(indd1, indd2))
print(index)

#----------------------------------------------------------------

indd1 <- index
d1 <- length(indd1)

}

#----------------------------------------------------------------

return(indd1)

}
