#PWLS algorithm: use CV to select tuning parameter
#unbalanced case

#-----------------------------------------------------------------

#library(geepack)
#library(lars)
#library(MASS)

#-----------------------------------------------------------------

#Argument:
#M -- n-dim vector containing #of obs for each subject
#X --- N*p design matrix
#Y --- N-dim responses
#corr --- working correlation: ind, exch, ar1
#gamma -- power of weight in adaLASSO
#link --- "Linear" linear model; "Logit" logistic regression; 
#         "Log" Poisson regression

#-----------------------------------------------------------------


PWLS_CV <- function(M, X, Y, corr, gamma, link) {

#-----------------------------------------------------------------

NGlob <- dim(X)[1]
pGlob <- dim(X)[2]
nGlob <- length(M)

#-----------------------------------------------------------------

#get the GEE estimator as the initial value 
#and weights in adaptive LASSO
datalist <- list(Y=Y, X=X)
Id <- rep(NA, NGlob)   #used to divide the subjects
Id[1:M[1]] <- rep(1, M[1])
for(i in 2:nGlob) {
Id[(sum(M[1:(i-1)])+1):(sum(M[1:i]))] <- rep(i, M[i])
}

if(link == "Linear") {
gee <- geese(Y~X-1, id=Id, data=datalist, 
             family=gaussian(), corstr=corr)
}

if(link == "Logit") {
gee <- geese(Y~X-1, id=Id, data=datalist, 
             family=binomial, corstr=corr)
}

if(link == "Log") {
gee <- geese(Y~X-1, id=Id, data=datalist, 
             family=poisson, corstr=corr)
}

beta_G <- gee$beta
tau <- gee$alpha

#-----------------------------------------------------------------

#construct new response Znew and covariates Xnew
Z <- rep(NA, NGlob)
Znew <- rep(NA, NGlob)
Xnew <- matrix(NA, NGlob, pGlob)
for(i in 1:nGlob) {
if(i == 1) {
aa <- 1
bb <- M[1]
}
if(i > 1) {
aa <- sum(M[1:(i-1)])+1
bb <- sum(M[1:i])
}
Xi <- X[aa:bb, ]
#-----------------------------------------------------------------
if(link == "Linear") {
mui <- Xi %*% beta_G
ai <- rep(1, M[i])
}
if(link == "Logit") {
mui <- mu_logit(Xi %*% beta_G)
ai <- as.vector(mui * (1 - mui))
}
if(link == "Log") {
mui <- mu_log(Xi %*% beta_G)
ai <- as.vector(mui)
}
#-----------------------------------------------------------------
#in the new quadratic form
#inverse of working correlation matrix  Ri
if(corr == "ind") { invRi <- diag(1, M[i]) }
if(corr == "exch") { invRi <- solve(CS(tau, M[i])) }
if(corr == "ar1") { invRi <- solve(AR1(tau, M[i])) }
#-----------------------------------------------------------------
if(M[i]==1) {
Ai <- ai
Di <- Ai*Xi
invAi <- 1/ai
invVi <- invAi*invRi
invVihalf <- sqrt(invVi)
}
if(M[i] > 1) {
Ai <- diag(ai)
Di <- Ai %*% Xi
invAihalf <- diag(1/sqrt(ai))
invVi <- invAihalf %*% invRi %*% invAihalf
EIGEN <- eigen(invVi)
invVihalf <- EIGEN$vectors %*% diag(sqrt(EIGEN$values)) %*% solve(EIGEN$vectors)
}
#-----------------------------------------------------------------
Si <- Y[aa:bb] - mui
Z[aa:bb] <- Di %*% beta_G - Si
#-----------------------------------------------------------------
Znew[aa:bb] <- invVihalf %*% Z[aa:bb]
Xnew[aa:bb, ] <- invVihalf %*% Di
}

#-----------------------------------------------------------------

#construct adjusted covariates Xadapt in adaLASSO 
Xadapt <- matrix(NA, NGlob, pGlob)
for(j in 1:pGlob) {
if(beta_G[j]!=0) {Xadapt[ ,j] <- ((abs(beta_G[j]))^gamma) * Xnew[ ,j]}
}

#-----------------------------------------------------------------

#adaptive LASSO
Xcenter <- scale(Xadapt, scale=FALSE)
Zcenter <- scale(Znew, scale=FALSE)
adalasso <- lars(Xcenter, Zcenter, type = "lasso", intercept = FALSE)
#use CV to select tuning parameter
lasso.s <- cv.lars(Xcenter, Zcenter, K=10, plot.it=F)
cvlasso.s <- which.min(lasso.s$cv)
optlasso.s <- lasso.s$index[cvlasso.s]
lasso.beta <- coef.lars(adalasso, s=optlasso.s, mode="fraction")
#return to the original estimate of paramters
dim.beta <- ncol(t(lasso.beta))
betahat <- rep(NA, dim.beta)
for(j in 1:dim.beta) {
 if(beta_G[j]!=0) {betahat[j] <- ((abs(beta_G[j]))^gamma) * lasso.beta[j]}
}

#-----------------------------------------------------------------

return(betahat)

}
