# main functions:
# G0stat  --- calculate GEE at beta=0, ie G0


#----------------------------------------------------------------

mu_logit <- function(x) {
mu <- exp(x) / (1+exp(x))
return(mu)
}

#----------------------------------------------------------------

mu_log <- function(x) {
mu <- exp(x)
return(mu)
}

#----------------------------------------------------------------

#exchangable correlation matrix
CS <- function(tau, m) {
if(m==1) {R <- 1}
if(m > 1) {
R <- diag(1, m)
for(i in 1:(m-1)) {
 for(j in (i+1):m) {
  R[i,j] <- R[j,i] <- tau
 }
}
}
return(R)
}

#----------------------------------------------------------------

#AR1 correlation matrix
AR1 <- function(tau, m) {
if(m==1) {R <- 1}
if(m > 1) {
R <- diag(1, m)
for(i in 1:(m-1)) {
 for(j in (i+1):m) {
  R[i,j] <- R[j,i] <- tau^(abs(i-j))
 }
}
}
return(R)
}


#----------------------------------------------------------------
#----------------------------------------------------------------

#Argument:
#M --- n-dim vector containing #of obs for each subject
#X --- N*p design matrix
#Y --- N-dim responses
#link --- "Linear" linear model; "Logit" logistic regression; 
#         "Log" Poisson regression
#tau --- estimate of parameters in R
#corr --- working correlation: ind, exch, ar1
#ind --- indices of potential important predictors 

#-----------------------------------------------------------------

#estimate of correlation coefficients
Tau <- function(M,X,Y,corr,ind,link) {

#-----------------------------------------------------------------

tau <- 1
NGlob <- dim(X)[1]  #total number of obs.
nGlob <- length(M)  #total number of subjects
Id <- rep(NA, NGlob)   #used to divide the subjects
Id[1:M[1]] <- rep(1, M[1])
for(i in 2:nGlob) {
Id[(sum(M[1:(i-1)])+1):(sum(M[1:i]))] <- rep(i, M[i])
}

#-----------------------------------------------------------------

if(link == "Linear") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
LM <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=LM, family=gaussian(), corstr=corr)
tau <- gee$alpha
}
}

#-----------------------------------------------------------------

if(link == "Logit") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
Logit <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=Logit, family=binomial, corstr=corr)
tau <- gee$alpha
}
}

#----------------------------------------------------------------

if(link == "Log") {
if(corr == "exch" | corr == "ar1") {
xx <- X[ ,ind]
Log <- list(y=Y, x=xx)
gee <- geese(y~x-1, id=Id, data=Log, family=poisson, corstr=corr)
tau <- gee$alpha
}
}

#-----------------------------------------------------------------

return(tau)

}


#-----------------------------------------------------------------


#p-dim estimating function G(0) value
G0stat <- function(M, X, Y, tau, corr, link) {

#----------------------------------------------------------------

nGlob <- length(M)  #total number of subjects

#----------------------------------------------------------------

if(link == "Linear") {

#----------------------------------------------------------------

Xi <- X[1:M[1], ]
Yi <- Y[1:M[1]]
if(corr == "ind") { Ri <- diag(1, M[1]) }
if(corr == "exch") { Ri <- CS(tau, M[1]) }
if(corr == "ar1") { Ri <- AR1(tau, M[1]) }
invRi <- solve(Ri)
if(M[1]==1) {G0 <- Xi %*% invRi %*% Yi}
if(M[1] > 1) {G0 <- t(Xi) %*% invRi %*% Yi}

#----------------------------------------------------------------

for(i in 2:nGlob) {
aa <- sum(M[1:(i-1)])+1
bb <- sum(M[1:i])
Xi <- X[aa:bb, ]
Yi <- Y[aa:bb]
if(corr == "ind") { Ri <- diag(1, M[i]) }
if(corr == "exch") { Ri <- CS(tau, M[i]) }
if(corr == "ar1") { Ri <- AR1(tau, M[i]) }
invRi <- solve(Ri)
if(M[i]==1) {G0 <- G0 + Xi %*% invRi %*% Yi}
if(M[i] > 1) {G0 <- G0 + t(Xi) %*% invRi %*% Yi}
}

G0 <- G0 / nGlob
}

#----------------------------------------------------------------

if(link == "Logit") {

#----------------------------------------------------------------

mu_c <- mu_logit(0)    #mean constant value at beta=0
sd_c <- sqrt(mu_c * (1-mu_c)) #sd constant value at beta=0
isd_c <- 1 / sd_c  #inverse of sd constant value at beta=0

#----------------------------------------------------------------

Xi <- X[1:M[1], ]
Yi <- Y[1:M[1]]
if(corr == "ind") { Ri <- diag(1, M[1]) }
if(corr == "exch") { Ri <- CS(tau, M[1]) }
if(corr == "ar1") { Ri <- AR1(tau, M[1]) }
invRi <- solve(Ri)
mui0 <- rep(mu_c,M[1])
Ai0_h <- diag(rep(sd_c, M[1]))   #Ai^{1/2}(0)
Ai0_invh <- diag(rep(isd_c, M[1]))  #Ai^{-1/2}(0)
G0 <- t(Xi) %*% Ai0_h %*% invRi %*% Ai0_invh %*% (Yi - mui0)

#----------------------------------------------------------------

for(i in 2:nGlob) {
aa <- sum(M[1:(i-1)])+1
bb <- sum(M[1:i])
Xi <- X[aa:bb, ]
Yi <- Y[aa:bb]
if(corr == "ind") { Ri <- diag(1, M[i]) }
if(corr == "exch") { Ri <- CS(tau, M[i]) }
if(corr == "ar1") { Ri <- AR1(tau, M[i]) }
invRi <- solve(Ri)
mui0 <- rep(mu_c,M[i])
Ai0_h <- diag(rep(sd_c, M[i]))  
Ai0_invh <- diag(rep(isd_c, M[i]))  
G0 <- G0 + t(Xi) %*% Ai0_h %*% invRi %*% Ai0_invh %*% (Yi - mui0)
}

G0 <- G0 / nGlob
}

#----------------------------------------------------------------

if(link == "Log") {

#----------------------------------------------------------------

mu_c <- mu_log(0)  #mean constant value at beta=0
sd_c <- sqrt(mu_c) #sd constant value at beta=0
isd_c <- 1 / sd_c  #inverse of sd constant value at beta=0

#----------------------------------------------------------------

Xi <- X[1:M[1], ]
Yi <- Y[1:M[1]]
if(corr == "ind") { Ri <- diag(1, M[1]) }
if(corr == "exch") { Ri <- CS(tau, M[1]) }
if(corr == "ar1") { Ri <- AR1(tau, M[1]) }
invRi <- solve(Ri)
mui0 <- rep(mu_c,M[1])
Ai0_h <- diag(rep(sd_c, M[1]))   #Ai^{1/2}(0)
Ai0_invh <- diag(rep(isd_c, M[1]))  #Ai^{-1/2}(0)
G0 <- t(Xi) %*% Ai0_h %*% invRi %*% Ai0_invh %*% (Yi - mui0)

#----------------------------------------------------------------

for(i in 2:nGlob) {
aa <- sum(M[1:(i-1)])+1
bb <- sum(M[1:i])
Xi <- X[aa:bb, ]
Yi <- Y[aa:bb]
if(corr == "ind") { Ri <- diag(1, M[i]) }
if(corr == "exch") { Ri <- CS(tau, M[i]) }
if(corr == "ar1") { Ri <- AR1(tau, M[i]) }
invRi <- solve(Ri)
mui0 <- rep(mu_c,M[i])
Ai0_h <- diag(rep(sd_c, M[i]))  
Ai0_invh <- diag(rep(isd_c, M[i]))  
G0 <- G0 + t(Xi) %*% Ai0_h %*% invRi %*% Ai0_invh %*% (Yi - mui0)
}

G0 <- G0 / nGlob
}

#----------------------------------------------------------------

return(G0) 

}
